import pandas as pd
from random import randint, choices, shuffle
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email import encoders
import logging
import datetime
import string
import os
import pdfkit
import smtplib
from email.utils import formataddr
import base64
import random

# Set up pdfkit configuration and logging
path_wkhtmltopdf = r'C:\Program Files\wkhtmltopdf\bin\wkhtmltopdf.exe'
config = pdfkit.configuration(wkhtmltopdf=path_wkhtmltopdf)
logging.basicConfig(filename='mail.log', level=logging.DEBUG)

def load_smtp_credentials(file_path):
    try:
        with open(file_path, 'r') as file:
            lines = file.readlines()
            credentials = []
            for line in lines:
                parts = line.strip().split(',')
                if len(parts) != 4:
                    raise ValueError("Each line must contain domain, email, app password, and port.")
                domain, email, password, port = parts
                credentials.append((domain, email, password, int(port)))
            return credentials
    except FileNotFoundError:
        logging.error(f"File '{file_path}' not found.")
        raise FileNotFoundError(f"File '{file_path}' not found.")
    except ValueError as ve:
        logging.error(f"Value error: {ve}")
        raise ValueError(f"Value error: {ve}")
    except Exception as e:
        logging.error(f"Unexpected error: {e}")
        raise Exception(f"Unexpected error: {e}")

smtp_credentials = load_smtp_credentials(os.getenv('SMTP_CREDENTIALS_PATH', 'smtp_credentials.txt'))

# Load data from CSV files and shuffle
emaildf = pd.read_csv('gmail.csv').sample(frac=1).reset_index(drop=True)
contactsData = pd.read_csv('contacts.csv').sample(frac=1).reset_index(drop=True)
subjects = pd.read_csv('subject.csv', encoding='latin1').sample(frac=1).reset_index(drop=True)
body_variations = pd.read_csv('body.csv', header=None, encoding='latin1')[0].tolist()

# Load products
def load_products(file_path):
    try:
        with open(file_path, 'r') as file:
            products = [line.strip() for line in file.readlines()]
        return products
    except FileNotFoundError:
        logging.error(f"File '{file_path}' not found.")
        raise FileNotFoundError(f"File '{file_path}' not found.")

products = load_products('products.txt')

# Shuffle body variations list
shuffle(body_variations)

# Define current date
current_date = datetime.datetime.now().strftime("%d-%m-%Y")

def encode_html_content(html_content):
    return base64.b64encode(html_content.encode('utf-8')).decode('utf-8')

def decode_html_content(encoded_content):
    return base64.b64decode(encoded_content.encode('utf-8')).decode('utf-8')

def generate_html_content(email, name, invoiceNo, charge, transaction_id, product_no, digi_no, tfn, current_date, product_name):
    with open('html_code.html', 'r') as html_file:
        html_content = html_file.read()
        html_content = html_content.replace('$email', email)
        html_content = html_content.replace('$name', name)
        html_content = html_content.replace('$invoice_no', invoiceNo)
        html_content = html_content.replace('$charge', str(charge))
        html_content = html_content.replace('$date', current_date)
        html_content = html_content.replace('$transactionid_no', str(transaction_id))
        html_content = html_content.replace('$product_no', str(product_no))
        html_content = html_content.replace('$digi_no', digi_no)
        html_content = html_content.replace('$tfn', tfn)
        html_content = html_content.replace('$product_name', product_name)
        html_content = html_content.replace('font-size: 12px;', 'font-size: 14px;')
    return encode_html_content(html_content)

def generate_invoice_no():
    return ''.join(choices(string.ascii_letters + string.digits, k=10))

def generate_digi_no():
    return ''.join(choices(string.ascii_letters + string.digits, k=20))

def generate_tfn():
    try:
        with open('phone_number.txt', 'r') as tfn_file:
            return tfn_file.read().strip()
    except FileNotFoundError:
        logging.error("File 'phone_number.txt' not found.")
        raise FileNotFoundError("File 'phone_number.txt' not found.")

def generate_pdf_from_html(html_content, pdf_file):
    decoded_html = decode_html_content(html_content)
    pdfkit.from_string(decoded_html, pdf_file, configuration=config)

def generate_body_message(name, invoice_no, charge):
    order_completion_message = choices(body_variations)[0]
    body_template = f"""Dear {name},

{order_completion_message}

Here’s a quick summary:

Invoice Number: {invoice_no}

Please find your invoice attached below.

If you have any questions, feel free to reach out to our support team. We’re here to help!

Thank you for choosing us!

Best regards,
Support Team"""
    return body_template

def create_message(sender_name, sender_email, recipient_name, recipient_email, subject, body_message, pdf_file):
    message = MIMEMultipart()
    message['From'] = formataddr((sender_name, sender_email))
    message['Subject'] = subject
    message['To'] = recipient_email

    with open(pdf_file, 'rb') as attachment:
        part = MIMEBase('application', 'octet-stream')
        part.set_payload(attachment.read())
    encoders.encode_base64(part)
    part.add_header('Content-Disposition', f'attachment; filename={pdf_file}')
    message.attach(part)
    message.attach(MIMEText(body_message, 'plain'))

    return message

def send_mail(sender_name, recipient_name, recipient_email, subjectWord, transaction_id, product_no, digi_no, current_date, smtp_domain, smtp_email, smtp_password, smtp_port):
    try:
        charge = randint(499, 799)
        invoiceNo = generate_invoice_no()
        tfn = generate_tfn()
        rand_string = ''.join(choices(string.ascii_uppercase, k=4))
        subject = f"{subjectWord} {rand_string}-{invoiceNo}"
        body_message = generate_body_message(recipient_name, invoiceNo, charge)
        
        # Select a random product
        selected_product = random.choice(products)
        
        html_content = generate_html_content(recipient_email, recipient_name, invoiceNo, charge, transaction_id, product_no, digi_no, tfn, current_date, selected_product)
        pdf_file = f'{rand_string}_{invoiceNo}.pdf'
        generate_pdf_from_html(html_content, pdf_file)
        message = create_message(sender_name, smtp_email, recipient_name, recipient_email, subject, body_message, pdf_file)

        with smtplib.SMTP(smtp_domain, smtp_port) as server:
            server.starttls()
            server.login(smtp_email, smtp_password)
            server.send_message(message)
        logging.info(f"Email sent successfully to {recipient_email} from {smtp_email}.")
        
        # Append to Excel sheet
        append_to_excel(recipient_email, recipient_name, invoiceNo, selected_product, charge)
        
        return True
    except Exception as e:
        logging.error(f"Error sending email: {e}")
        return False
    finally:
        if os.path.exists(pdf_file):
            os.remove(pdf_file)

def append_to_excel(email, name, invoice_no, product_name, amount):
    data = {
        'Email Address': [email],
        'Name': [name],
        'Invoice Number': [invoice_no],
        'Product Name': [product_name],
        'Amount': [amount]
    }
    
    df = pd.DataFrame(data)
    
    # Append to existing Excel file or create it if it doesn't exist
    file_path = 'email_log.xlsx'
    if os.path.exists(file_path):
        with pd.ExcelWriter(file_path, mode='a', engine='openpyxl', if_sheet_exists='overlay') as writer:
            df.to_excel(writer, index=False, header=False, startrow=writer.sheets['Sheet1'].max_row)
    else:
        df.to_excel(file_path, index=False)

def start_mail_system(emaildf, subjects):
    totalSend = 0
    num_senders = len(emaildf)
    num_credentials = len(smtp_credentials)
    email_index = 0
    credential_index = 0

    for i in range(len(contactsData)):
        sender = emaildf.iloc[email_index % num_senders]
        sender_name = sender['sender_name']
        smtp_domain, smtp_email, smtp_password, smtp_port = smtp_credentials[credential_index % num_credentials]
        recipient_name = contactsData.iloc[i]['name']
        recipient_email = contactsData.iloc[i]['email']
        subjectWord = subjects.iloc[i % len(subjects)]['subject']  # Ensure subject is cycled
        transaction_id = randint(100, 999)
        product_no = randint(10000, 99999)
        digi_no = generate_digi_no()
        current_date = datetime.datetime.now().strftime("%d-%m-%Y")

        # Send email and update status
        email_sent = send_mail(
            sender_name,
            recipient_name,
            recipient_email,
            subjectWord,
            transaction_id,
            product_no,
            digi_no,
            current_date,
            smtp_domain,
            smtp_email,
            smtp_password,
            smtp_port
        )

        if email_sent:
            totalSend += 1
            print(f"Email sent to: {recipient_name} ({recipient_email})")

        email_index += 1
        credential_index += 1

    print(f"\nTotal emails sent: {totalSend}")

try:
    start_mail_system(emaildf, subjects)
except KeyboardInterrupt:
    logging.info("\n\nCode stopped by user")
    print("\nCode stopped by user.")
except Exception as e:
    logging.error(f"An error occurred: {e}", exc_info=True)
    print(f"An error occurred: {e}")
